using System;

public class UnmanagedResourceHolder : IDisposable
{
  bool m_isValid = true;

  public void Close()
  {
    Console.WriteLine("Close.");
    
    Dispose();
  }

  public void Dispose()
  {
    Console.WriteLine("Dispose.");

    // stop GC from calling destructor
    GC.SuppressFinalize(this);

    Dispose(true);
  }

  ~UnmanagedResourceHolder()
  {
    Console.WriteLine("Destructor.");

    Dispose(false);
  }

  protected virtual void Dispose(bool disposing)
  {
    Console.WriteLine("Dispose(bool).");

    // protect against multiple threads
    lock (this)
    {
      // execute only if not called
      //    from destructor
      if (disposing)
      {
        Console.WriteLine("Dispose(true).");

        // call other object members of
        //  this type that need to be disposed
      }

      if (m_isValid)
      {
        Console.WriteLine("m_isValid == true.");

        // dispose of unmanaged resources
        //  that are still valid
        m_isValid = false;
      }
    }
  }
}

class UsingStatement
{
  static void Main()
  {
    using (UnmanagedResourceHolder resHold = new UnmanagedResourceHolder())
    {
      // do something with resHold
    }

    Console.ReadLine();
  }
}
