using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace WinDataGridDemo
{
	/// <summary>
	/// Summary description for DataGridCombo.
	/// </summary>
	public class DataGridCombo : WinDataGridDemo.BaseForm
	{
		const int CountryCol = 4;

		ComboBox cbxCountries = new ComboBox();

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public DataGridCombo()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose (bool disposing)
		{
			if (disposing)
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			((System.ComponentModel.ISupportInitialize)(this.dgrCustomers)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.dataSet1)).BeginInit();
			// 
			// dgrCustomers
			// 
			this.dgrCustomers.Name = "dgrCustomers";
			this.dgrCustomers.Click += new System.EventHandler(this.dgrCustomers_Click);
			this.dgrCustomers.CurrentCellChanged += new System.EventHandler(this.dgrCustomers_CurrentCellChanged);
			this.dgrCustomers.Paint += new System.Windows.Forms.PaintEventHandler(this.dgrCustomers_Paint);
			this.dgrCustomers.Scroll += new System.EventHandler(this.dgrCustomers_Scroll);
			// 
			// dataSet1
			// 
			this.dataSet1.DataSetName = "NewDataSet";
			this.dataSet1.Locale = new System.Globalization.CultureInfo("en-US");
			// 
			// DataGridCombo
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(432, 326);
			this.Name = "DataGridCombo";
			this.Text = "DataGridCombo";
			this.Load += new System.EventHandler(this.DataGridCombo_Load);
			((System.ComponentModel.ISupportInitialize)(this.dgrCustomers)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.dataSet1)).EndInit();
		}
		#endregion
		
		private void DataGridCombo_Load(object sender, System.EventArgs e)
		{
			cbxCountries.Visible = false;
            cbxCountries.Items.Clear();
			cbxCountries.TextChanged += new EventHandler(cbxCountries_TextChanged);

			string[] countries = GetCountries();
			cbxCountries.Items.AddRange(countries);

            dgrCustomers.PreferredRowHeight = cbxCountries.Height;
			dgrCustomers.Controls.Add(cbxCountries);
		}

		private string[] GetCountries()
		{
			ArrayList countries = new ArrayList();
			DataTable customers = dataSet1.Tables["Customers"];

			foreach (DataRow row in customers.Rows)
			{
				countries.Add(row["Country"]);
			}

			return (string[])countries.ToArray(typeof(string));
		}

		// load cell with what was selected in combo box
		private void cbxCountries_TextChanged(object sender, System.EventArgs e)
		{
			DataGridCell currCell = dgrCustomers.CurrentCell;

			if (currCell.ColumnNumber == CountryCol)
			{
				cbxCountries.Visible = false;

				dgrCustomers[currCell] = cbxCountries.Text;
			}
		}

		// make sure combo box width is set properly on repaint
		private void dgrCustomers_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			if (dgrCustomers.CurrentCell.ColumnNumber == CountryCol)
			{
				cbxCountries.Width = dgrCustomers.GetCurrentCellBounds().Width;
			}
		}

		// hides combo box when it's being
		//	worked on or should not be shown
		private void HideComboBox()
		{
			cbxCountries.Visible = false;
			cbxCountries.Width   = 0;
		}

		// initialize combo box
		private void dgrCustomers_CurrentCellChanged(object sender, System.EventArgs e)
		{
			HideComboBox();

			DataGridCell currCell = dgrCustomers.CurrentCell;

			if (currCell.ColumnNumber == CountryCol)
			{
				Rectangle cellBounds = dgrCustomers.GetCurrentCellBounds();

				cbxCountries.Left    = cellBounds.Left;
				cbxCountries.Top     = cellBounds.Top;
				cbxCountries.Text    = (string)dgrCustomers[currCell];
				cbxCountries.Visible = true;
			}
		}
		
		private void dgrCustomers_Click(object sender, System.EventArgs e)
		{
			HideComboBox();
		}
		
		private void dgrCustomers_Scroll(object sender, System.EventArgs e)
		{
			HideComboBox();
		}
	}
}
